/*
    ES Knot
    Copyright 2005, 2006 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifndef __GLMATH_H
#define __GLMATH_H

#ifdef USE_OES_2
#include <GLES2/gl2.h>
#else
#include <GLES/gl.h>
#endif

#ifndef USE_OES_2
#define USING_FIXED
#endif

#ifdef USING_FIXED
#define F(x) ((GLfixed)((x)*(1<<16)))
#define TOFLOAT(x) (float(x)/(1<<16))
#define FIXED_TO_NATIVE(x) (x)
#define TYPE GLfixed
#define TYPETOKEN GL_FIXED

#define glClearColor glClearColorx
#define glClearDepth glClearDepthx
#define glTranslate glTranslatex
#define glLightv glLightxv
#define glMaterialv glMaterialxv
#define glMaterial glMaterialx
#define glFrustum glFrustumx
#define glColor4 glColor4x
#define glRotate glRotatex
#define glTranslate glTranslatex
#define glLineWidth glLineWidthx
#define glDepthRange glDepthRangex
#define glGetTypev glGetFixedv
#define GLGETTYPEV_NAME "glGetFixedv"
#define glMultMatrix glMultMatrixx
#define glLoadMatrix glLoadMatrixx
#define glOrtho glOrthox
#define glLightModelv glLightModelxv

#define MULT(a,b) (((a)>>8)*((b)>>8))

#else

#define F(x) float(x)
#define TOFLOAT(x) float(x)
#define FIXED_TO_NATIVE(x) (float(x)/(1<<16))
#define TYPE GLfloat
#define TYPETOKEN GL_FLOAT

#define glClearColor glClearColor
#define glClearDepth glClearDepthf
#define glTranslate glTranslatef
#define glLightv glLightfv
#define glMaterialv glMaterialfv
#define glMaterial glMaterialf
#define glFrustum glFrustumf
#define glColor4 glColor4f
#define glRotate glRotatef
#define glTranslate glTranslatef
#define glLineWidth glLineWidth
#define glDepthRange glDepthRangef
#define glGetTypev glGetFloatv
#define GLGETTYPEV_NAME "glGetFloatv"
#define glMultMatrix glMultMatrixf
#define glLoadMatrix glLoadMatrixf
#define glOrtho glOrthof
#define glLightModelv glLightModelfv

#define MULT(a,b) ((a)*(b))

#endif

#ifdef GL_ES_VERSION_2_0
#define glTexParameter glTexParameteri
#else
#define glTexParameter glTexParameterx
#endif

#define printVect(v) printf("[%.3f %.3f %.3f %.3f]\n", TOFLOAT(v[0]), TOFLOAT(v[1]), TOFLOAT(v[2]), TOFLOAT(v[3]))
#define printMatrix(m)			\
	do {				\
		printf("[%.3f %.3f %.3f %.3f]\n", TOFLOAT(m[0]), TOFLOAT(m[4]), TOFLOAT(m[8]), TOFLOAT(m[12]));		\
		printf("[%.3f %.3f %.3f %.3f]\n", TOFLOAT(m[1]), TOFLOAT(m[5]), TOFLOAT(m[9]), TOFLOAT(m[13]));		\
		printf("[%.3f %.3f %.3f %.3f]\n", TOFLOAT(m[2]), TOFLOAT(m[6]), TOFLOAT(m[10]), TOFLOAT(m[14]));	\
		printf("[%.3f %.3f %.3f %.3f]\n", TOFLOAT(m[3]), TOFLOAT(m[7]), TOFLOAT(m[11]), TOFLOAT(m[15]));	\
	} while (0)

#endif
