/*
    ES Knot
    Copyright 2009 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifndef __KNOT2_H
#define __KNOT2_H

#include "knotinterface.h"

#include <GLES2/gl2.h>

#ifdef RENAME_OES_2_CLASS
#define KNOT_CLASS_NAME Knot2
#define RADIAL_BLUR_CLASS_NAME RadialBlur2
#define MOTION_BLUR_CLASS_NAME MotionBlur2
#else
#define KNOT_CLASS_NAME Knot
#define RADIAL_BLUR_CLASS_NAME RadialBlur
#define MOTION_BLUR_CLASS_NAME MotionBlur
#endif

#include "glmath.h"

#ifdef RINGS
#undef RINGS
#endif
#ifdef POINTS
#undef POINTS
#endif
#define RINGS 70
#define POINTS 8

#define SINETABLESIZE 512
#define SINETABLEMASK 0x01FF
#define SINETABLEMASKSIZE 9

class RADIAL_BLUR_CLASS_NAME;
class MOTION_BLUR_CLASS_NAME;

class KNOT_CLASS_NAME : public KnotInterface {
public:
	KNOT_CLASS_NAME(int w, int h);
	~KNOT_CLASS_NAME();
	void draw(unsigned int msec);
	void reinit();
	void clearContext();
	void setSize(int w, int h);

	void setTexMode(TexMode mode);
	void setUpdateVertices(bool update);
	void setMotionBlur(bool blur);
	void setRadialBlur(bool blur);
	void setRadialGlow(bool glow);

	void startMorph();

	void enableRotate(bool rotate);
	void enableGravity(bool gravity);
	void rotate(int x, int y);
	
	void setGravity(float x, float y, float z);

	bool getMatrixSupported();

private:
	TYPE vertices[3*(RINGS+1)*(POINTS+1)];
	TYPE normals[3*(RINGS+1)*(POINTS+1)];
	TYPE texcoords[2*(RINGS+1)*(POINTS+1)];
	GLushort indices[6*RINGS*POINTS];
	TYPE ringvertices[4*POINTS];
	TYPE ringcoords23[16*RINGS];
	TYPE ringcoords25[16*RINGS];
	TYPE *ringcoords;
	TYPE ringcoordstemp[16*RINGS];

	static void polar(float pos, TYPE *vect, int P, int Q);
	void initProjection();
	void precalc();
	void genTexture();
	void genCelTexture();
	void calcDynVertices();
	void calcDynVerticesNormals(TYPE *verts, TYPE *norms, const TYPE *coords);
	void makeRingCoords(TYPE *coords, int P, int Q);

	TYPE fastSin(TYPE x);
	void morphMatrices(TYPE *out, const TYPE *first, const TYPE *second, TYPE pos);
	
	void rotateGravity();

	void reinit(TexMode mode);
	int width, height;

	unsigned int lasttime;
	bool morphing;
	unsigned int morphStart;

	GLuint texture;
	GLuint celTexture;
	TexMode texMode;
	TYPE texOffsetY;

	bool updateBumps;
	TYPE bumpOffset;

	TYPE sinTable[SINETABLESIZE];

	RADIAL_BLUR_CLASS_NAME* radialBlur;
	bool doRadialBlur;

	MOTION_BLUR_CLASS_NAME* motionBlur;
	bool doMotionBlur;

	bool rotating;
	
	bool gravityEnabled;
	bool gravitySet;
	TYPE curGravity[3];
	TYPE targetGravity[3];

	TYPE complementVect[3];
	TYPE extraRotMatrix[16];
	bool extraRotInited;

	float projMatrix[16];
	float modelViewMatrix[16];
	GLuint untexturedProg, untexturedVert, untexturedFrag;
	GLuint texSpecProg, texSpecVert, texSpecFrag;
	GLuint outlineProg, outlineVert, outlineFrag;
	GLuint celShadeProg, celShadeVert, celShadeFrag;

};

#endif
